import os

from ament_index_python.packages import get_package_share_directory

from launch import LaunchDescription
from launch.actions import DeclareLaunchArgument, ExecuteProcess
from launch.substitutions import LaunchConfiguration, TextSubstitution

from launch_ros.actions import Node

from launch.actions import GroupAction
from launch_ros.actions import PushRosNamespace
from launch_ros.actions import SetParameter
from launch.actions import IncludeLaunchDescription
from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.conditions import IfCondition

def generate_launch_description():
    ns_arg = DeclareLaunchArgument("ns", default_value=TextSubstitution(text="/ardu0"))
    name_arg = DeclareLaunchArgument("name", default_value=TextSubstitution(text="ardu0"))
    location_arg = DeclareLaunchArgument("location", default_value=TextSubstitution(text="granso"))
    mqtt_host_arg = DeclareLaunchArgument("mqtt_host", default_value=TextSubstitution(text="localhost"))    
    passwd_arg = DeclareLaunchArgument("password", default_value=TextSubstitution(text="QWERTY"))
    gazebo_arg = DeclareLaunchArgument("gazebo", default_value='false')
    wara_arg = DeclareLaunchArgument("wara", default_value='false')
    show_tst_arg = DeclareLaunchArgument("show_tst", default_value='false')
    type_arg = DeclareLaunchArgument("type", default_value=TextSubstitution(text="arducopter"))
    prefix3_arg = DeclareLaunchArgument("prefix3", default_value=TextSubstitution(text="air"))    
    prefix4_arg = DeclareLaunchArgument("prefix4", default_value=TextSubstitution(text="simulation"))    
    dronekit_connect_string_arg = DeclareLaunchArgument("dronekit_connect_string", default_value=TextSubstitution(text="udp:127.0.0.1:14551"))    
    
    exec_include = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(
            os.path.join(
                get_package_share_directory('lrs_exec'),
                'exec.launch.py')),
        launch_arguments={"ns": LaunchConfiguration('ns'),
                          "location": LaunchConfiguration('location'),
                          "basic_executors": "false"}.items()
    )

    tst_streamer_include = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(
            os.path.join(
                get_package_share_directory('lrs_exec'),
                'tst_streamer.launch.py')),
        launch_arguments={"ns": LaunchConfiguration('ns'),
                          "display_type": "exec",
                }.items()
    )

#    mqtt_include = IncludeLaunchDescription(
#        PythonLaunchDescriptionSource(
#            os.path.join(
#                get_package_share_directory('lrs_mqtt'),
#                'gazebo_bridge.launch.py')),
#        launch_arguments={"ns": LaunchConfiguration('ns'),
#                          "mqtt_host": LaunchConfiguration('mqtt_host')}.items(),
#        condition=IfCondition(LaunchConfiguration('gazebo'))
#    )
#
    wara_include = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(
            os.path.join(
                get_package_share_directory('lrs_json_api'),
                'wara.launch.py')),
        launch_arguments={"ns": LaunchConfiguration('ns'),
                          "password": LaunchConfiguration('password'),
                          "type": LaunchConfiguration('type'),
                          "prefix3": LaunchConfiguration('prefix3'),
                          "prefix4": LaunchConfiguration('prefix4'),
        }.items(),
        condition=IfCondition(LaunchConfiguration('wara'))        
    )

    ardupilot_node = Node(
        package='lrs_ardupilot',
        output='screen',
        emulate_tty=True,            
        namespace=LaunchConfiguration('ns'),
        executable='exec_ardupilot_executors',
    )

    command_node = Node(
        package='lrs_ardupilot',
        output='screen',
        emulate_tty=True,            
        namespace=LaunchConfiguration('ns'),
        executable='ardupilot_command',
    )

    rqt_image_view_node = Node(
        package='rqt_image_view',
        output='screen',
        emulate_tty=True,            
        executable='rqt_image_view',
        namespace=LaunchConfiguration('ns'),
        arguments=[[LaunchConfiguration('ns'), '/tststream_exec/image_raw/compressed']],
        condition=IfCondition(LaunchConfiguration('show_tst'))                
        )

    rqt_image_view_exec = ExecuteProcess(
         cmd=[[
             'ros2 run rqt_image_view rqt_image_view ',
             LaunchConfiguration('ns'),
             '/tststream_exec/image_raw/compressed'
             ]],
         shell=True
    )

    return LaunchDescription([
        dronekit_connect_string_arg,
        prefix3_arg,
        prefix4_arg,
        type_arg,
        wara_arg,
        gazebo_arg,
        mqtt_host_arg,
        ns_arg,
        name_arg,
        location_arg,
        passwd_arg,
        show_tst_arg,
        exec_include,
        tst_streamer_include,
#        mqtt_include,
        wara_include,
#        ardupilot_node,
        rqt_image_view_node,
        ardupilot_node
    ])    
