import collections.abc
import collections
collections.MutableMapping = collections.abc.MutableMapping

import rclpy
from rclpy.node import Node
from rclpy.executors import MultiThreadedExecutor
from rclpy.callback_groups import ReentrantCallbackGroup

from dronekit import connect, VehicleMode
from lrs_ardupilot.ardupilot_connect import ArduPilotConnect

from lrs_ardupilot.executors.take_off import TakeoffExecutorFactory
from lrs_ardupilot.executors.land import LandExecutorFactory
from lrs_ardupilot.executors.in_air_test import InAirTestExecutorFactory
from lrs_ardupilot.executors.fly_to import FlyToExecutorFactory

from lrs_ardupilot.executors.in_air_goal import InAirGoalExecutorFactory
from lrs_ardupilot.executors.move_to import MoveToExecutorFactory
from lrs_ardupilot.executors.return_home import ReturnHomeExecutorFactory
from lrs_ardupilot.executors.go_home import GoHomeExecutorFactory

def main(args=None):
    name = "ardupilot"
    
    print(f'Hi from {name}.')

    rclpy.init(args=args)

    parameter_node = Node("parameter_node")
    parameter_node.declare_parameter("dronekit_connect_string", "udp:127.0.0.1:14551")
    connection_string = parameter_node.get_parameter("dronekit_connect_string").value

    executor = MultiThreadedExecutor(num_threads=8)

    print(f'Connecting to vehicle on: {connection_string}')

    connection_string = "tcp:mavproxy:14551"
    vehicle = connect(connection_string, wait_ready=True)
    
    
    ardupilot_connect = ArduPilotConnect(vehicle)
    executor.add_node(ardupilot_connect)    

    #
    # Exec ececutors
    #

    node = TakeoffExecutorFactory(vehicle)
    executor.add_node(node)
   
    node = LandExecutorFactory(vehicle)
    executor.add_node(node)
   
    node = InAirTestExecutorFactory(vehicle)
    executor.add_node(node)
   
    node = FlyToExecutorFactory(vehicle)
    executor.add_node(node)

    # Expansion nodes
    
    node = InAirGoalExecutorFactory()
    executor.add_node(node)
   
    node = MoveToExecutorFactory()
    executor.add_node(node)
   
    node = ReturnHomeExecutorFactory()
    executor.add_node(node)
   
    node = GoHomeExecutorFactory()
    executor.add_node(node)
   
    print(f"Spinning {name}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    

