import os
import threading
import rclpy
from rclpy.node import Node
from rclpy.executors import MultiThreadedExecutor
from rclpy.callback_groups import ReentrantCallbackGroup

from lrs_exec.exec_util import get_executor_factory_class_objects


from lrs_msgs_common.msg import TakeoffInfo
from std_msgs.msg import UInt8, Float32, String
from sensor_msgs.msg import NavSatFix, Joy
from geometry_msgs.msg import QuaternionStamped


from lrs_exec.executors.add_task_to_team_queue import AddTaskToTeamQueueExecutorFactory
from lrs_exec.executors.input_handler import InputHandlerExecutorFactory
from lrs_exec.executors.say import SayExecutorFactory
from lrs_exec.executors.string_pub import StringPubExecutorFactory
from lrs_exec.executors.noop import NoopExecutorFactory
from lrs_exec.executors.fail import FailExecutorFactory
from lrs_exec.executors.send_signal import SendSignalExecutorFactory
#from lrs_exec.executors.start_data_recording import StartDataRecordingExecutorFactory
#from lrs_exec.executors.stop_data_recording import StopDataRecordingExecutorFactory
from lrs_exec.executors.supp import SuppExecutorFactory
from lrs_exec.executors.wait import WaitExecutorFactory
from lrs_exec.executors.wait_joy import WaitJoyExecutorFactory
from lrs_exec.executors.send_signal import SendSignalExecutorFactory
from lrs_exec.executors.query import QueryExecutorFactory
from lrs_exec.executors.remote import RemoteExecutorFactory

from lrs_ardupilot.executors.take_off import TakeoffExecutorFactory
from lrs_ardupilot.executors.land import LandExecutorFactory
from lrs_ardupilot.executors.fly_to import FlyToExecutorFactory
from lrs_ardupilot.executors.move_to import MoveToExecutorFactory
from lrs_ardupilot.executors.in_air_goal import InAirGoalExecutorFactory
from lrs_ardupilot.executors.in_air_test import InAirTestExecutorFactory

def main(args=None):
    name = "exec_ardupilot_executors"
    
    print(f'Hi from {name}.')

    rclpy.init(args=args)

    executor = MultiThreadedExecutor(num_threads=18)

    #
    # Exec executors
    #

    node = NoopExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = AddTaskToTeamQueueExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = InputHandlerExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = SayExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = StringPubExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = FailExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = SuppExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = WaitExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = WaitJoyExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = SendSignalExecutorFactory()
    executor.add_node(node)
    #coll.add(node)
    
    node = QueryExecutorFactory()
    executor.add_node(node)
    #coll.add(node)

    node = RemoteExecutorFactory()
    executor.add_node(node)
    #coll.add(node)


    #
    # Ardupilot Executors
    #

    node = TakeoffExecutorFactory()
    executor.add_node(node)
    
    node = LandExecutorFactory()
    executor.add_node(node)
    
    node = FlyToExecutorFactory()
    executor.add_node(node)
    
    node = MoveToExecutorFactory()
    executor.add_node(node)
    
    node = InAirGoalExecutorFactory()
    executor.add_node(node)
    
    node = InAirTestExecutorFactory()
    executor.add_node(node)
    
    
    print(f"Spinning {name}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
