import time
import rclpy

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

from lrs_exec.command import InAirCheckClient

class InAirTestExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id)
        self.client = InAirCheckClient(node)        
        
    def do_work(self):
        try:
            self.node.get_logger().info(f'{__name__} do_work')
            #self.params.print()

            self.client.send_request()
            
            while not self.client.result:
                time.sleep(0.1)
            
            self.client.send_request()
            
            while not self.client.result:
                time.sleep(0.1)

            answer = "unknown"
            if self.client.result.success:
                if self.client.result.result:
                    answer = "true"
                else:
                    answer = "false"                    
            
            response = self.set_node_parameter_string(self.id, "answer", answer)
            self.node.get_logger().info(f'end in_air_test: {answer}')                    
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION land: {type(exc)} - {exc}')
        self.do_work_finished()                
            
class InAirTestExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("in-air-test")
        
    def get_executor(self, node, id):
        return self.add_executor(InAirTestExecutor(node, id))
    
