import os
import time
import rclpy

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

from lrs_exec.command import LandActionClient

class LandExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id)
        self.add_resource_to_lock("fly")
        self.action_client = LandActionClient(node, callback_group=node.group)        
        self.node.get_logger().info(f'{__name__} created')

    def do_work(self):
        try:
            self.node.get_logger().info(f'{__name__} do_work')

            #self.params.print()

            self.action_client.send_goal()            
            self.action_client.send_goal()

            counter = 0
            while not self.action_client.have_goal_response and counter < 100:
                counter += 1
                time.sleep(0.1)

            self.action_client.send_goal()

            counter = 0
            while not self.action_client.have_goal_response and counter < 100:
                counter += 1
                time.sleep(0.1)

            if not self.action_client.have_goal_response:
                self.fail(f'Manager execution failed in do_work_begin: No goal response in 10 seconds')
                self.do_work_finished()
                return

            if not self.action_client.goal_accepted:
                self.fail(f'Manager execution failed in do_work_begin: Goal not accepted')
                self.do_work_finished()
                return
                
            while not self.action_client.result:
                time.sleep(0.1)

            self.do_work_finished()
            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION land: {type(exc)} - {exc}')
                
            
class LandExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("land")

    def get_executor(self, node, id):
        return self.add_executor(LandExecutor(node, id))
        
