import os
import time

from lrs_util.coordtrans import CoordTrans
from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.executor import Executor
from lrs_util.markerutil import *
from lrs_msgs_common.msg import WaypointsInfo

class MissionExecutor(TickedExecutor):
    def __init__(self, node, id, blocking=False):
        super().__init__(node, id, blocking=blocking)
        self.ma_pub = self.node.create_publisher(MarkerArray, "/visualization_marker_array", 10, callback_group=self.node.group)
        self.ext_ma_pub = self.node.create_publisher(MarkerArray, "/ext/visualization_marker_array", 10, callback_group=self.node.group)
        self.waypoints_pub = self.node.create_publisher(WaypointsInfo, "sensor/waypoints", 10, callback_group=self.node.group)        
        self.coordtrans = CoordTrans(location=os.environ.get("LRS_LOCATION"))

    def same_position(self, cgp, gp):
        res = False
        xy_distance = self.coordtrans.wgs84_distance(cgp.longitude, cgp.latitude, gp.longitude, gp.latitude)
        z_distance = abs(gp.altitude - cgp.altitude)
        print("XYDISTANCE:", xy_distance)
        print("ZDISTANCE:", z_distance)
        if (xy_distance < 1.0) and (z_distance < 1.0):
            res = True
        return res

    def pub_waypoints(self, gps):
        msg = WaypointsInfo()
        msg.waypoints = gps
        self.waypoints_pub.publish(msg)

    def display_path(self, gps):
        marker_path = []
        for el in gps:
            marker_path.append(self.coordtrans.wgs84_to_world(el.longitude, el.latitude, el.altitude))
        
        ns = self.node.get_namespace()
        marr = get_marker_array_3d(marker_path, colindex=0, marker_ns=f'{ns.lstrip("/")}-fly-to', scale=1.0)
        self.ma_pub.publish(marr)
        self.ext_ma_pub.publish(marr)


    def handle_signals(self):
        if self.have_signal():
            signal = self.pop_signal()
            self.node.get_logger().info(f'Executor got signal: {signal}')
            if signal == "$finished":
                self.signal_finish()
                self.do_work_finished()
                return
            if signal == "$enough":
                self.signal_goal_client.send_request(self.goal_uuid, "$enough")
                self.do_work_finished()
                return
            if signal == "$abort":
                self.signal_goal_client.send_request(self.goal_uuid, "$abort")                    
                self.abort_fail("Got abort signal")
                self.do_work_finished()
            if signal == "$pause":
                self.signal_goal_client.send_request(self.goal_uuid, "$pause")
            if signal == "$continue":
                self.signal_goal_client.send_request(self.goal_uuid, "$continue")                    
        
