#!/usr/bin/env python3

from os import system, environ, getlogin, getenv
import time
import sys
import os

from tmux import *

from optparse import OptionParser
parser = OptionParser()
parser.add_option ("-a", "--attach", action="store_true", dest="attach", help="Attach")
parser.add_option ("", "--ns", action="store", type="string", dest="ns", default="/ardu0", help="Namespace to use (default /husky0 namespace)")
parser.add_option ("", "--world", action="store", type="string", dest="world", default="maze", help="World to use (default maze)")
parser.add_option ("", "--stop", action="store_true", dest="stop", help="Stop execution.")
parser.add_option ("", "--sim", action="store_true", dest="sim", help="Use Gazebo Sim.")
parser.add_option ("", "--slam", action="store_true", dest="slam", help="Use Slam.")
parser.add_option ("", "--vicon", action="store_true", dest="vicon", help="Use Vicon and real Turtlebot 4.")
parser.add_option ("", "--no-vicon", action="store_true", dest="novicon", help="Use Vicon and real Turtlebot 4.")
parser.add_option ("", "--laser", action="store_true", dest="laser", help="Use Laser for Pointcloud.")
parser.add_option ("", "--depth", action="store_true", dest="depth", help="Use ASUS Xtion for Pointcloud.")
parser.add_option ("", "--t4depth", action="store_true", dest="t4depth", help="Use ASUS Xtion for Pointcloud.")
parser.add_option ("", "--dynamic-objects", action="store_true", dest="dynamic_objects", help="Use Laser for Pointcloud.")

parser.add_option ("", "--camera", action="store_true", dest="camera", help="Use Laser for Pointcloud.")
parser.add_option ("", "--jazzy", action="store_true", dest="jazzy", help="jassy.")

parser.add_option ("", "--wara", action="store_true", dest="wara", help="Connect to the specified waraps broker. Start the json api bridge.)")
parser.add_option ("", "--main-wara", action="store_true", dest="main_wara", help="Connect to main waraps broker. Start the json api bridge.)")
parser.add_option ("", "--location", action="store", type="string", dest="location", default=f'{os.getenv("LRS_LOCATION")}', help=f'Location to use (default: {os.getenv("LRS_LOCATION")})')
parser.add_option ("", "--scale", action="store", type="int", dest="scale", default=1, help="Location to use")
parser.add_option ("", "--tststream", action="store_true", dest="tststream", help="Stream tst")
parser.add_option ("", "--wara-broker-host", action="store", type="string", dest="wara_broker_host", default="broker.waraps.org", help="Wara broker (default broker.waraps.org)")
parser.add_option ("", "--wara-broker-port", action="store", type="string", dest="wara_broker_port", default="8883", help="Wara broker port (default 8883)")
parser.add_option ("", "--wara-broker-user", action="store", type="string", dest="wara_broker_user", default="", help="Wara broker user")
parser.add_option ("", "--wara-broker-password", action="store", type="string", dest="wara_broker_password", default="", help="Wara broker passwd")
parser.add_option ("", "--octree-filename", action="store", type="string", dest="octree_filename", default=f'{os.getenv("HOME")}/lrs2_ws/src/lrs_granso/octomaps/granso_l0_center_12.5cm.bt', help="")
parser.add_option ("", "--omehost", action="store", type="string", dest="omehost", default="ome.waraps.org", help="Ome host (default ome.waraps.org)")
parser.add_option ("", "--color", action="store_true", dest="color", help="Stream color camera")
parser.add_option ("", "--bitrate", action="store", type="string", dest="bitrate", default="1000k", help="Bitrate ffmpeg (default 1000k)")
parser.add_option ("", "--omews", action="store", type="string", dest="omews", default="wss://ome.waraps.org:3334/", help="Ome ws (default wss://ome.waraps.org:3334/)")

parser.add_option ("", "--external", action="store_true", dest="external", default=False, help="Push to aiics broker (default=False)")
(options, args) = parser.parse_args()

def get_list_param(name, items):
    res = f"{name}:='["
    first_item = True
    for item in items:
        if first_item:
            res += f'\\\"{item}\\\"'
            first_item = False
        else:
            res += f', \\\"{item}\\\"'
    res += "]'"
    return res

def ffmpeg_command(name, fifo, resolution="1024x600", pixfmt="bgr24", omehost="ome.waraps.org"):
    command = f'while [ true ] ; do ffmpeg -f rawvideo -r 15 -vcodec rawvideo -pix_fmt {pixfmt} -s {resolution} -i {fifo} -r 15 -c:v libx264 -b:v {options.bitrate} -bufsize {options.bitrate} -pix_fmt yuv420p -tune zerolatency -preset ultrafast -f flv rtmp://{omehost}/app/{name} ; sleep 2; done'
    return command

def main_unit(ns):
    if ns:
        command = f"new-session -n MISC -d -s {ns} 'exec bash'"
    else:
        command = f"new-session -n MISC -d -s NOAMESPACE 'exec bash'"        
    tmux(command)

    if options.vicon:
        ros2("vicon", "ros2 launch lrs_vicon vicon.launch.py rate:=10")
    if options.jazzy:        
        ros2("mavproxy", f"cd; source venv/bin/activate; mavproxy.py --master=tcp:localhost:14552 --out=tcpin:mavproxy:14553", run=False)
    else:
        ros2("mavproxy", f"mavproxy.py --master=tcp:localhost:14552 --out=tcpin:mavproxy:14553", run=False)
        
    if options.jazzy:
        ros2("ardu_manager", f"cd; source venv/bin/activate; cd {os.getenv("HOME")}/lrs2_ws/src/lrs_ardupilot/lrs_ardupilot;  python ardu_manager.py --ros-args -r __ns:={ns} -p port:=14552")
    else:
        pass
        # ros2("ardu_manager", f"cd /home/tompe/lrs2_ws/src/lrs_ardupilot/lrs_ardupilot; source venv/bin/activate; python ardu_manager.py --ros-args -r __ns:={ns} -p port:=14552")
        
    ros2("ardu_command", f"ros2 run lrs_ardupilot ardu_command --ros-args -r __ns:={ns}")

    external = options.external    
    
    ros2("exec", f"ros2 launch lrs_exec exec.launch.py location:={options.location} ns:={ns} external:={external} basic_executors:=false tree_executor:=false")

    ros2("tree_executor_bt", f'cd /home/tompe/lrs2_ws/src/lrs_py_trees/lrs_py_trees; source venv/bin/activate; python3 tree_executor.py  --ros-args -r __ns:={ns}')

    ros2("take_off", f'ros2 run lrs_exec tst_command --ros-args -r __ns:={ns} -p unit:={ns} -p command:=take-off -p bt:=true', run=False)

    ros2("land", f'ros2 run lrs_exec tst_command --ros-args -r __ns:={ns} -p unit:={ns} -p command:=land -p bt:=true', run=False)
    
    ros2("move_to", f'ros2 run lrs_exec tst_command --ros-args -r __ns:={ns} -p unit:={ns} -p command:=move-to -p x:=10.0 -p y:=100.0 -p z:=40.0 -p speed:=5.0 -p bt:=true', run=False)

    tmux(f'attach-session -t {ns}')
    
if __name__ == "__main__":
    ns = options.ns

    main_unit(ns)    
